// Bankrun based on Gertler and  Kiyotaki  2015 with housing and borrowers
// After  run
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 var 
// Lagrangians,*******1-3
lambda_P,lambda_I,lambda_m,

//consumption 4-7
C_P,C_I,C_B,C,

//housing 8-9
h_P,h_I,

//Deposit & mortgage  10-11
D,M

//capital  12-16
K_h,K_b,K,i_k,p_k,

//wage  17-20
w_P,w_I,l_P,l_I,

//interests & price  21-24
r,r_b,p_h,xi,

//BANK
  // networth 25-28
N_l, N,  A, N_n,
  //multiplier 29-32
phi_d, phi_k, phi, s,

//cost function 33-34
f_k,f_k_1,

//Total factor productivity 35
Z,

// GDP 36
Y,
//Run 37-38
X,run,

//adjustment cost 39-44
AC_P,AC_I,AC_B,ac_dh,ac_kh,ac_mh, %

//results  45-62
data_Y, data_C , data_C_P, data_C_I, data_C_B,
data_h_P,data_h_I, data_p_h,
data_K, data_K_h , data_K_b, data_i_k , data_p_k,
data_A , data_M , data_D ,
spread_rb_r ,data_phi,data_Z;

%********************************
 varexo epsilon_z;
%********************************

  parameters
  //beta
beta_P,beta_I,beta_B,
 // utility coeficient
varphi_c,varphi_h,varphi_l,
//depreciation
delta_k,
//probability of life
sigma,
//downpayment
theta_m,
//coeficient of default
theta_b,
//donation
varpi,
//cost function
alpha_k, 
//firms
alpha_f, iota_P, iota_I, Z_bar,
//shock
rho_z, sigma_eps, 
//adjustment cost
psi_dh,psi_db,psi_mh,psi_mb,psi_kb,psi_kh,psi_k;

%********************************
// define the parameters
 %% Run steadystate_Finder before run dynare file

load Parameters.mat;
for i=1:length(M_.params)-9
    deep_parameter_name = M_.param_names(i,:);
    eval(['M_.params(i)  = ' deep_parameter_name ' ;'])
end;
//shock
rho_z = 0.95 ; 
sigma_eps = rho_z^(121-1)*0.01;
//adjustment cost
psi_dh = 0;
psi_db = 0;
psi_mh = 0;
psi_mb = 0;
psi_kb = 0;
psi_kh = 0;
psi_k  = 2.0;

%********************************

model;

// Model local variables:every time this variable appears in the model, Dynare will
//substitute it by the expression assigned to the variable used outside.
#  Y_bar=steady_state(Y);
#  C_bar=steady_state(C);
#  C_P_bar=steady_state(C_P);
#  C_I_bar=steady_state(C_I);
#  C_B_bar=steady_state(C_B);

#  h_P_bar=steady_state(h_P);
#  h_I_bar=steady_state(h_I);
#  p_h_bar=steady_state(p_h);

#  K_bar=steady_state(K);
#  K_h_bar=steady_state(K_h);
#  K_b_bar=steady_state(K_b);
#  i_k_bar=steady_state(i_k);
#  p_k_bar=steady_state(p_k);

#  A_bar=steady_state(A);
#  M_bar=steady_state(M);
#  D_bar=steady_state(D);

#  phi_bar=steady_state(phi);

%*******************************

// Lagrangian 
lambda_P = (1- varphi_c)/(C_P - varphi_c * C_P(-1)); //1 lambda_P
lambda_I = (1- varphi_c)/(C_I - varphi_c * C_I(-1)); //2 lambda_I

//Houshold budget constraints
  //lender budget 3
C_P + D + p_h * (h_P - h_P(-1)) + p_k * (K_h-(1-delta_k)* K_h(-1)) + f_k + (1-sigma) * N_n = (1) * w_P * l_P + (1+r(-1)) *D(-1) + 1*xi * K_h(-1) - AC_P ; //3.C_P

  //borrower budget 4
C_I + p_h * (h_I - h_I(-1)) + (1+r_b)    *M(-1)= (1) * w_I* l_I + M - AC_I; //4 C_I

//FOCs
  // Lender respect to h^p 5
p_h =  varphi_h / (lambda_P * h_P)+ beta_P *  (lambda_P(1)/ lambda_P) * p_h(1);//5 p_h

  // Lender respect to D 6
1 + ac_dh = beta_P *  (lambda_P(1)/ lambda_P) * (1+r); //6 r

  // Lender respect to K^h 7
p_k + f_k_1 + ac_kh = beta_P * (lambda_P(1)/ lambda_P) * (p_k(1)* (1- delta_k)+ 1*xi(1)); //7 K_h
// cost function 
f_k   = alpha_k/ 2 * (K_h)^2; //8
f_k_1 = alpha_k * K_h;// 9


  // borrower respect to h^I 
(1-  (lambda_m / lambda_I) * theta_m ) * p_h=  varphi_h/ (lambda_I * h_I) + beta_I * ( lambda_I(1)/ lambda_I) * p_h(1); //10 lambda_m

  // borrower respect to M 
1- (lambda_m/lambda_I) +ac_mh = beta_I *( lambda_I(1)/ lambda_I) * (1+r_b(1)); //11  r_b

// colateral constraint 
M = theta_m * p_h * h_I ; //12 M

 
// Aggregate banking 
   //Arbitrage 
 1*xi + (1- delta_k)* p_k = (1+r_b)    * p_k(-1); //13 xi
 
A = p_k * K_b + M ;  //14 K_b

   // living networth 14
 N_l = (1+r_b) * A(-1) - (1+r(-1)) * D(-1); //15  N_l

   // Total networth 15
N = sigma * N_l + (1- sigma) * N_n; //16 N

   // Bnak consumption  
C_B = (1- sigma) * N_l; //17 C_B
   // budget constraint 
C_B + A + AC_B = D + N_l + (1- sigma) * N_n ; //18 phi


   // internal leverage ratio 

theta_b * ( phi_d + 1) = beta_B * ((1- sigma)+ sigma * (theta_b * ( phi_d(1) + 1))) * (s(1) * phi_d + (1+r_b(1))) ; //19 phi_d
   // phi 
phi = A/N ; //20  A
phi_d = D/N ;//21 D
phi_k = p_k * K_b /N; //22 phi_k
s =  r_b - r(-1) ; //23 s

// capital producer 
(1 - psi_k * (i_k/i_k(-1) - 1)^2 /2) * i_k = K - (1-delta_k) * K(-1); //24 i_k
p_k * (1 - psi_k *(i_k/i_k(-1)-1)*(i_k/i_k(-1)) - psi_k/2 *(i_k/i_k(-1)-1)^2) + beta_P *((lambda_P(1)/lambda_P) * psi_k * p_k(1) * (i_k(1)/i_k-1)*(i_k(1)/i_k)^2) = 1;//25

//Firms
 alpha_f * Y/K(-1) = 1 * xi ; // 26 xi
(1-alpha_f)* iota_P * Y/l_P = 1 * w_P ; //27
(1-alpha_f)* iota_I * Y/l_I = 1 * w_I ; //28

  // Lender respect to l^P 8
 varphi_l/(1-l_P) = lambda_P * (1) * w_P; //29 w_P

  // borrower respect to l^I 11
varphi_l/ (1-l_I)= lambda_I * (1) * w_I; //30 l^I


//Market clearing
1 = h_P + h_I; //31 h_p
K = K_b + K_h; //32
   //****// Walras !! Y = 1*(xi * K(-1) + w_P * l_P + w_I * l_I) ;////*****
//Y = f_k + C + i_k ; % 33
Y = Z * K(-1)^alpha_f * (l_P^iota_P*l_I^iota_I)^(1-alpha_f) ;
C = C_P + C_I + C_B ;//34 C

//start up funds
N_n = varpi * Y(-1) ; //35

//shock 
%z = rho_z * z(-1) - epsilon_z ;    //                    %Productivity shock  
%Z = Z_bar * exp(z); //
(Z-Z_bar)/Z_bar = rho_z * (Z(-1)-Z_bar)/Z_bar - epsilon_z; //36

// Run 
X = ((1 + r_b)* phi(-1))/((1 + r(-1))* phi_d(-1)); //37
run = 1 - X ; //38

// Adjustment cost  39-44
AC_P=(psi_dh/2)* (D-D(-1))^2/D_bar + (psi_kh/2)* (K_h-K_h(-1))^2/K_h_bar;//
AC_I=(psi_mh/2)* (M-M(-1))^2/M_bar ;//
AC_B=(psi_db/2)* (D-D(-1))^2/D_bar + (psi_kb/2)* (K_b-K_b(-1))^2/K_b_bar +(psi_mb/2)* (M-M(-1))^2/M_bar;//
ac_dh= (psi_dh)* (D-D(-1))/D_bar ;//
ac_kh= (psi_kh)* (K_h-K_h(-1))/K_h_bar;//
ac_mh= (psi_mh)* (M-M(-1))/M_bar ;//


%/*
//Results  45-54
data_Y = 100*(Y/Y_bar - 1); //
data_C = 100*(C/C_bar - 1) ; //
data_C_I = 100*(C_I/C_I_bar - 1) ; //
data_C_P = 100*(C_P/C_P_bar - 1) ; //
data_C_B = 100*(C_B/C_B_bar - 1) ; //

data_h_P = 100*(h_P/h_P_bar - 1) ; //
data_h_I = 100*(h_I/h_I_bar - 1) ; //
data_p_h = 100*(p_h/p_h_bar - 1) ; //

data_K   = 100*(K/K_bar - 1) ; //
data_K_h = 100*(K_h/K_h_bar - 1) ; //
data_K_b = 100*(K_b/K_b_bar - 1) ; //
data_i_k = 100*(i_k/i_k_bar - 1) ; //
data_p_k = 100*(p_k/p_k_bar - 1) ; //

data_A = 100*(A/A_bar - 1) ; //
data_M = 100*(M/M_bar - 1)  ; //
data_D = 100*(D/D_bar - 1) ; //


spread_rb_r = 4* (r_b - r(-1)); //
data_phi = 100*(phi/phi_bar - 1) ; //
data_Z = 100*(Z/Z_bar - 1) ; //
%*/

end;
%********************************

load Var_at_run.mat;

initval;
lambda_P = lambda_P_run; lambda_I= lambda_I_run;
C_P=C_P_run; C_I=C_I_run ;C_B=C_B_run ;C=C_run ; 
h_P=h_P_run ;h_I=h_I_run ;
l_P=l_P_run; l_I=l_I_run;  D=D_run; M=M_run; K_h=K_h_run; K_b=K_b_run;K=K_run;
w_P=w_P_run; w_I=w_I_run; r=r_run; r_b=r_b_run; p_h=p_h_run; p_k=p_k_run; 
N_l=N_l_run;  N=N_run; phi_d=phi_d_run; phi=phi_run;  phi_k=phi_k_run;  s=s_run; 
i_k=i_k_run; f_k=f_k_run ; xi=xi_run; A=A_run;  Y=Y_run; 

X=X_run; Z = Z_run; %N_n=varpi * Y_run;
data_Y=data_Y_run; data_C=data_C_run; data_C_P=data_C_P_run; data_C_I=data_C_I_run; data_C_B=data_C_B_run;
data_h_P=data_h_P_run; data_h_I=data_h_I_run;  data_p_h=data_p_h_run;
data_K=data_K_run; data_K_h=data_K_h_run;  data_K_b=data_K_b_run; data_i_k=data_i_k_run; data_p_k=data_p_k_run;
data_A=data_A_run; data_M=data_M_run; data_D=data_D_run;
spread_rb_r=spread_rb_r_run;  data_phi=data_phi_run;
end;
%********************************


load just_for_check_0_BR.mat
endval;

lambda_P = lambda_P_0; lambda_I= lambda_I_0;
C_P=C_P_0; C_I=C_I_0 ;C_B=C_B_0 ;C=C_0 ; 
h_P=h_P_0 ;h_I=h_I_0 ;
l_P=l_P_0; l_I=l_I_0;  D=D_0; M=M_0; K_h=K_h_0;K_b=K_b_0; K=K_0;
w_P=w_P_0; w_I=w_I_0; r=r_0; r_b=r_b_0; p_h=p_h_0; p_k=p_k_0; 
N_l=N_l_0;  N=N_0; phi_d=phi_d_0; phi=phi_0;  phi_k=phi_k_0;  s=s_0; 
i_k=i_k_0; f_k=f_k_0 ; xi=xi_0; A=A_0;  Y=Y_0; 

X=X_0;run=run_0; Z = Z_bar; N_n=N_n_0;


end;


%steady;

check;

%shocks;
%var epsilon_z;
%periods 1;
%values (sigma_eps);
%var epsilon_z ; stderr sigma_eps; 
%end;

// order 1=  At first order they are deviations from the steady state.
// with relative_irf= relative_irf only normalizes the variables by the shock size so that the shock sizes are comparable. It does not rescale the other variables by their steady state values to make them percentage deviations!
// order 2 et + = % deviation 
//stoch_simul(periods=120,irf = 100,order =1);
//simul(periods=500,stack_solve_algo = 0);

perfect_foresight_setup(periods=50);
options_.simul.robust_lin_solve=1;
load Var_at_run.mat;
oo_.endo_simul(:,1)=variables_at_run;
perfect_foresight_solver(stack_solve_algo =6); %lmmcp








